from    Debug            import Debug


class RLInput(Debug):
    # PROVIDES_* variables (override if needed):
    PROVIDES_READ    = ["album",
                        "artist",
                        "bitrate",
                        "duration",
                        "elapsed_time",
                        "file_size",
                        "genre",
                        "last_played",
                        "play_count",
                        "playing",
                        "rating",
                        "repeat",
                        "shuffle",
                        "title",
                        "track_number",
                        "uri"]
    PROVIDES_WRITE   = ["elapsed_time",
                        "repeat",
                        "shuffle",
                        "playing",
                        "rating"]
    PROVIDES_COMMAND = ["NEXT",
                        "PAUSE",
                        "PLAY",
                        "PLAY_PAUSE",
                        "PLAY_URI",
                        "PREVIOUS"]

    # class-wide variables
    _callbacks_active    = []
    _callbacks_song        = []
    _callbacks_state    = []


    def __init__(self):
        pass


    def add_callback_active(self, callback):
        """
        Add callback to internal callbacks list.
        """

        if (callback not in self._callbacks_active):
            self.debug("Adding callback (active): %s::%s" % (callback.__module__, callback.__name__))

            self._callbacks_active.append(callback)

            # Run callback immediately for good measure.
            self._run_callbacks([callback])
        #end if

        return(True)
    #end add_callback_active()

    def add_callback_song(self, callback):
        """
        Add callback to internal callbacks list.
        """

        if (callback not in self._callbacks_song):
            self.debug("Adding callback (song): %s::%s" % (callback.__module__, callback.__name__))

            self._callbacks_song.append(callback)

            # Run callback immediately for good measure.
            self._run_callbacks([callback])
        #end if

        return(True)
    #end add_callback_song()

    def add_callback_state(self, callback):
        """
        Add callback to internal callbacks list.
        """

        if (callback not in self._callbacks_state):
            self.debug("Adding callback (state): %s::%s" % (callback.__module__, callback.__name__))

            self._callbacks_state.append(callback)

            # Run callback immediately for good measure.
            self._run_callbacks([callback])
        #end if

        return(True)
    #end add_callback_state()


    def clear_callbacks_active(self):
        """
        Delete all (in)?active callback from internal callbacks list.
        """

        self._callbacks_active = []

        return(True)
    #end clear_callbacks_state()

    def clear_callbacks_song(self):
        """
        Delete all song callbacks from internal callbacks list.
        """

        self._callbacks_song = []

        return(True)
    #end clear_callbacks_song()

    def clear_callbacks_state(self):
        """
        Delete all state callbacks from internal callbacks list.
        """

        self._callbacks_state = []

        return(True)
    #end clear_callbacks_state()


    def _run_callbacks(self, list):
        """
        Run all callbacks in list.
        """

        for callback in list:
            if (callable(callback)):
                #self.debug("Running callback (%s::%s)" % (callback.__module__, callback.__name__))
                
                try:
                    if (callable(callback)):
                        callback(self)
                        #self.add_thread(callback, self)
                    #end if
                except Exception, error:
                    self.handle_exception(error)
                #end try
            #end if
        #end for
    #end _run_callbacks()


    def active_change(self, obj_ref=None):
        """
        Process callbacks on active/inactive change.
        """

        self._run_callbacks(self._callbacks_active)
    #end song_change()

    def song_change(self, obj_ref=None):
        """
        Process callbacks on song change.
        """

        self._run_callbacks(self._callbacks_song)
    #end song_change()

    def state_change(self, obj_ref=None):
        """
        Process callbacks on state change.
        """

        self._run_callbacks(self._callbacks_state)
    #end state_change()


    def __getitem__(self, attribute):
        """
        Override standard __getitem__ method.
        """

        print("Not implemented.")

        return(None)
    #end __getitem__()

    def __setitem__(self, attribute, value):
        """
        Override standard __setitem__ method.
        """

        print("Not implemented.")

        return(None)
    #end __setitem__()


    def run_command(self, command):
        """
        Run specified command.
        """

        print("Not implemented.")

        return(None)
    #end run_command()


    def disable(self):
        print("Not implemented.")

        return(False)
    #end disable()

    def enable(self):
        print("Not implemented.")

        return(False)
    #end enable()
#end class
